@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -52,6 +24,7 @@ package com.openexchange.gdpr.dataexport.impl;
 import static com.openexchange.gdpr.dataexport.impl.DataExportUtility.stringFor;
 import static com.openexchange.groupware.upload.impl.UploadUtility.getSize;
 import static com.openexchange.java.Autoboxing.I;
+import static com.openexchange.java.Autoboxing.L;
 import java.time.DayOfWeek;
 import java.util.ArrayList;
 import java.util.Calendar;
@@ -696,7 +669,7 @@ public class DataExportServiceImpl implements DataExportService {
         }
 
         cancelAllTimerTasks(true);
-        stopProcessingTasks(false);
+        stopProcessingTasks(false, true);
     }
 
     /**
@@ -766,41 +739,40 @@ public class DataExportServiceImpl implements DataExportService {
             DayOfWeekTimeRanges dayOfWeekTimeRanges = rangesOfTheWeek.get(dayOfWeekFor(dayOfTheWeek));
             if (dayOfWeekTimeRanges != null) {
                 // Today... Check if current time is included in any time range; otherwise determine closest
-                TimeRange closest = null;
-                for (Iterator<TimeRange> it = dayOfWeekTimeRanges.getRanges().iterator(); closest == null && it.hasNext();) {
+                TimeRange applicableTimeRange = null;
+                Iterator<TimeRange> it = dayOfWeekTimeRanges.getRanges().iterator();
+                while (applicableTimeRange == null && it.hasNext()) {
                     TimeRange timeRange = it.next();
-                    if (timeRange.contains(time)) {
-                        // Already in a defined time range from today, hence start immediately
-                        Calendar tmp = Calendar.getInstance();
-                        tmp.setTimeInMillis(currentTimeMillis);
-                        tmp.set(Calendar.HOUR_OF_DAY, timeRange.getEnd().getHour());
-                        tmp.set(Calendar.MINUTE, timeRange.getEnd().getMinute());
-                        tmp.set(Calendar.SECOND, timeRange.getEnd().getSecond());
-                        long stopDelay = tmp.getTimeInMillis() - currentTimeMillis;
-                        schedule(0, stopDelay, currentTimeMillis, timerService);
-                        return;
-                    }
-
-                    if (time.compareTo(timeRange.getStart()) < 0) {
-                        closest = timeRange;
+                    if (timeRange.contains(time) || (time.compareTo(timeRange.getStart()) < 0)) {
+                        applicableTimeRange = timeRange;
                     }
                 }
 
-                if (closest != null) {
+                if (applicableTimeRange != null) {
                     // Schedule for closest time range today
                     Calendar tmp = Calendar.getInstance();
                     tmp.setTimeInMillis(currentTimeMillis);
-                    tmp.set(Calendar.HOUR_OF_DAY, closest.getStart().getHour());
-                    tmp.set(Calendar.MINUTE, closest.getStart().getMinute());
-                    tmp.set(Calendar.SECOND, closest.getStart().getSecond());
-                    long startDelay = tmp.getTimeInMillis() - currentTimeMillis;
 
-                    tmp.set(Calendar.HOUR_OF_DAY, closest.getEnd().getHour());
-                    tmp.set(Calendar.MINUTE, closest.getEnd().getMinute());
-                    tmp.set(Calendar.SECOND, closest.getEnd().getSecond());
-                    long stopDelay = tmp.getTimeInMillis() - currentTimeMillis;
+                    long startDelay;
+                    if (applicableTimeRange.contains(time)) {
+                        // Already in a defined time range from today, hence start immediately
+                        startDelay = 0L;
+                    } else {
+                        tmp.set(Calendar.HOUR_OF_DAY, applicableTimeRange.getStart().getHour());
+                        tmp.set(Calendar.MINUTE, applicableTimeRange.getStart().getMinute());
+                        tmp.set(Calendar.SECOND, applicableTimeRange.getStart().getSecond());
+                        startDelay = tmp.getTimeInMillis() - currentTimeMillis;
+                    }
 
-                    schedule(startDelay, stopDelay, currentTimeMillis, timerService);
+                    // Determine stop delay
+                    Long stopDelay = determineStopDelay(applicableTimeRange, it, rangesOfTheWeek, dayOfTheWeek, tmp, currentTimeMillis);
+                    if (stopDelay == null) {
+                        // Infinite
+                        schedule(startDelay, Optional.empty(), currentTimeMillis, timerService);
+                        return;
+                    }
+
+                    schedule(startDelay, Optional.of(stopDelay), currentTimeMillis, timerService);
                     return;
                 }
 
@@ -819,7 +791,8 @@ public class DataExportServiceImpl implements DataExportService {
             }
 
             // Schedule for first time range of follow-up day
-            TimeRange firstOnFollowUpDay = dayOfWeekTimeRanges.getRanges().get(0);
+            Iterator<TimeRange> it = dayOfWeekTimeRanges.getRanges().iterator();
+            TimeRange firstOnFollowUpDay = it.next();
 
             Calendar tmp = Calendar.getInstance();
             tmp.setTimeInMillis(currentTimeMillis);
@@ -829,65 +802,146 @@ public class DataExportServiceImpl implements DataExportService {
             tmp.set(Calendar.SECOND, firstOnFollowUpDay.getStart().getSecond());
             long startDelay = tmp.getTimeInMillis() - currentTimeMillis;
 
-            tmp.set(Calendar.HOUR_OF_DAY, firstOnFollowUpDay.getEnd().getHour());
-            tmp.set(Calendar.MINUTE, firstOnFollowUpDay.getEnd().getMinute());
-            tmp.set(Calendar.SECOND, firstOnFollowUpDay.getEnd().getSecond());
-            long stopDelay = tmp.getTimeInMillis() - currentTimeMillis;
+            // Determine stop delay
+            Long stopDelay = determineStopDelay(firstOnFollowUpDay, it, rangesOfTheWeek, dayOfTheWeek, tmp, currentTimeMillis);
+            if (stopDelay == null) {
+                // Infinite
+                schedule(startDelay, Optional.empty(), currentTimeMillis, timerService);
+                return;
+            }
 
-            schedule(startDelay, stopDelay, currentTimeMillis, timerService);
+            schedule(startDelay, Optional.of(stopDelay), currentTimeMillis, timerService);
         } catch (RuntimeException e) {
             throw DataExportExceptionCode.UNEXPECTED_ERROR.create(e, e.getMessage());
         }
     }
 
-    private void schedule(long startDelay, long stopDelay, long currentTimeMillis, TimerService timerService) {
+    private Long determineStopDelay(TimeRange applicableTimeRange, Iterator<TimeRange> remainingRanges, Map<DayOfWeek, DayOfWeekTimeRanges> rangesOfTheWeek, int dayOfTheWeek, Calendar tmp, long currentTimeMillis) {
+        // Determine end. First consume remaining time ranges
+        TimeRange endTimeRange = applicableTimeRange;
+        boolean checkNextDays = true;
+        while (remainingRanges.hasNext()) {
+            TimeRange timeRange = remainingRanges.next();
+            if (endTimeRange.getEnd().isEndBeginningOfOtherDay(timeRange.getStart())) {
+                endTimeRange = timeRange;
+            } else {
+                checkNextDays = false;
+            }
+        }
+
+        // Now, check for follow-up day of week
+        boolean infinite = false;
+        int dayDiffer = 0;
+        if (checkNextDays) {
+            int nextDayOfWeek = dayOfTheWeek + 1;
+            if (nextDayOfWeek > Calendar.SATURDAY) {
+                nextDayOfWeek = Calendar.SUNDAY;
+            }
+            boolean lookUp = true;
+            for (DayOfWeekTimeRanges nextDayOfWeekTimeRanges; lookUp && (nextDayOfWeekTimeRanges = rangesOfTheWeek.get(dayOfWeekFor(nextDayOfWeek))) != null;) {
+                boolean first = true;
+                for (Iterator<TimeRange> nextit = nextDayOfWeekTimeRanges.getRanges().iterator(); lookUp && nextit.hasNext();) {
+                    TimeRange nextTimeRange = nextit.next();
+                    if (endTimeRange.getEnd().isEndBeginningOfOtherDay(nextTimeRange.getStart())) {
+                        if (nextDayOfWeek == dayOfTheWeek) {
+                            // Full round-trip
+                            infinite = true;
+                            lookUp = false;
+                        }
+                        if (first) {
+                            dayDiffer++;
+                            first = false;
+                        }
+                        endTimeRange = nextTimeRange;
+                    } else {
+                        lookUp = false;
+                    }
+                }
+                nextDayOfWeek = nextDayOfWeek + 1;
+                if (nextDayOfWeek > Calendar.SATURDAY) {
+                    nextDayOfWeek = Calendar.SUNDAY;
+                }
+            }
+        }
+
+        if (infinite) {
+            return null;
+        }
+
+        if (dayDiffer > 0) {
+            tmp.add(Calendar.DAY_OF_YEAR, dayDiffer);
+        }
+        tmp.set(Calendar.HOUR_OF_DAY, endTimeRange.getEnd().getHour());
+        tmp.set(Calendar.MINUTE, endTimeRange.getEnd().getMinute());
+        tmp.set(Calendar.SECOND, endTimeRange.getEnd().getSecond());
+        return L(tmp.getTimeInMillis() - currentTimeMillis);
+    }
+
+    private void schedule(long startDelay, Optional<Long> optionalStopDelay, long currentTimeMillis, TimerService timerService) {
         if (startDelay < 0) {
             throw new IllegalArgumentException("Start delay must not be less than 0 (zero): " + startDelay);
         }
-        if (stopDelay < startDelay) {
-            throw new IllegalArgumentException("Stop delay (" + stopDelay + ") must be greater than start delay (" + startDelay + ")");
-        }
-        final long endTimeMillis = currentTimeMillis + stopDelay;
-        Runnable startTask = new Runnable() {
 
-            @Override
-            public void run() {
-                try {
-                    if (System.currentTimeMillis() < endTimeMillis) {
-                        startProcessingTasks();
+        if (optionalStopDelay.isPresent()) {
+            long stopDelay = optionalStopDelay.get().longValue();
+            if (stopDelay < startDelay) {
+                throw new IllegalArgumentException("Stop delay (" + stopDelay + ") must be greater than start delay (" + startDelay + ")");
+            }
+            final long endTimeMillis = currentTimeMillis + stopDelay;
+            Runnable startTask = new Runnable() {
+
+                @Override
+                public void run() {
+                    try {
+                        if (System.currentTimeMillis() < endTimeMillis) {
+                            startProcessingTasks();
+                        }
+                    } catch (Exception e) {
+                        LOG.error("Failed to start data export tasks", e);
                     }
-                } catch (Exception e) {
-                    LOG.error("Failed to start data export tasks", e);
                 }
-            }
-        };
-        this.periodicStartTask.set(timerService.scheduleWithFixedDelay(startTask, startDelay, config.getCheckForTasksFrequency(), TimeUnit.MILLISECONDS));
+            };
+            this.periodicStartTask.set(timerService.scheduleWithFixedDelay(startTask, startDelay, config.getCheckForTasksFrequency(), TimeUnit.MILLISECONDS));
 
-        Runnable stopTask = new Runnable() {
+            Runnable stopTask = new Runnable() {
 
-            @Override
-            public void run() {
-                try {
-                    stopProcessingTasks(true);
-                } catch (Exception e) {
-                    LOG.error("Failed to stop data export tasks", e);
+                @Override
+                public void run() {
+                    try {
+                        stopProcessingTasks(true, false);
+                    } catch (Exception e) {
+                        LOG.error("Failed to stop data export tasks", e);
+                    }
                 }
-            }
-        };
-        this.stopTask.set(timerService.schedule(stopTask, stopDelay, TimeUnit.MILLISECONDS));
+            };
+            this.stopTask.set(timerService.schedule(stopTask, stopDelay, TimeUnit.MILLISECONDS));
 
-        Runnable nextScheduleRun = new Runnable() {
+            Runnable nextScheduleRun = new Runnable() {
 
-            @Override
-            public void run() {
-                try {
-                    planSchedule();
-                } catch (Exception e) {
-                    LOG.error("Failed to schedule data export tasks", e);
+                @Override
+                public void run() {
+                    try {
+                        planSchedule();
+                    } catch (Exception e) {
+                        LOG.error("Failed to schedule data export tasks", e);
+                    }
                 }
-            }
-        };
-        this.nextRunTask.set(timerService.schedule(nextScheduleRun, stopDelay + 60000L, TimeUnit.MILLISECONDS));
+            };
+            this.nextRunTask.set(timerService.schedule(nextScheduleRun, stopDelay + 60000L, TimeUnit.MILLISECONDS));
+        } else {
+            Runnable startTask = new Runnable() {
+
+                @Override
+                public void run() {
+                    try {
+                        startProcessingTasks();
+                    } catch (Exception e) {
+                        LOG.error("Failed to start data export tasks", e);
+                    }
+                }
+            };
+            this.periodicStartTask.set(timerService.scheduleWithFixedDelay(startTask, 0, config.getCheckForTasksFrequency(), TimeUnit.MILLISECONDS));
+        }
 
         LOG.debug("Scheduled execution of data export tasks on this node");
     }
@@ -903,9 +957,8 @@ public class DataExportServiceImpl implements DataExportService {
     }
 
     private void cancelTimerTasks(AtomicReference<ScheduledTimerTask> timerTaskReference, boolean mayInterruptIfRunning, boolean purge) {
-        ScheduledTimerTask timerTask = timerTaskReference.get();
+        ScheduledTimerTask timerTask = timerTaskReference.getAndSet(null);
         if (timerTask != null) {
-            timerTaskReference.set(null);
             timerTask.cancel(mayInterruptIfRunning);
         }
 
@@ -1013,17 +1066,17 @@ public class DataExportServiceImpl implements DataExportService {
     }
 
     private boolean isConsiderPermissionDeniedErrors(DataExportJob job) throws OXException {
-        boolean addDiagnosticsReport = false;
+        boolean considerPermissionDeniedErrors = false;
         ConfigViewFactory viewFactory = services.getOptionalService(ConfigViewFactory.class);
         if (viewFactory != null) {
             DataExportTask task = job.getDataExportTask();
             ConfigView view = viewFactory.getView(task.getUserId(), job.getDataExportTask().getContextId());
-            addDiagnosticsReport = ConfigViews.getDefinedBoolPropertyFrom("com.openexchange.gdpr.dataexport.considerPermissionDeniedErrors", addDiagnosticsReport, view);
+            considerPermissionDeniedErrors = ConfigViews.getDefinedBoolPropertyFrom("com.openexchange.gdpr.dataexport.considerPermissionDeniedErrors", considerPermissionDeniedErrors, view);
         }
-        return addDiagnosticsReport;
+        return considerPermissionDeniedErrors;
     }
 
-    synchronized void stopProcessingTasks(boolean cancelStartStopTimerTasks) {
+    synchronized void stopProcessingTasks(boolean cancelStartStopTimerTasks, boolean onShutDown) {
         if (cancelStartStopTimerTasks) {
             cancelTimerTasks(periodicStartTask, false, false);
             cancelTimerTasks(stopTask, false, false);
@@ -1033,9 +1086,11 @@ public class DataExportServiceImpl implements DataExportService {
             }
         }
 
-        for (Map.Entry<Future<Void>, DataExportTaskExecution> executionEntry : executions.entrySet()) {
-            executionEntry.getValue().stop();
-            executionEntry.getKey().cancel(true);
+        if (onShutDown || config.isAllowPausingRunningTasks()) {
+            for (Map.Entry<Future<Void>, DataExportTaskExecution> executionEntry : executions.entrySet()) {
+                executionEntry.getValue().stop();
+                executionEntry.getKey().cancel(true);
+            }
         }
     }
 
