@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -91,6 +63,7 @@ import com.openexchange.gdpr.dataexport.DataExportWorkItem;
 import com.openexchange.gdpr.dataexport.impl.DataExportLock;
 import com.openexchange.gdpr.dataexport.impl.DataExportUtility;
 import com.openexchange.java.ConvertUtils;
+import com.openexchange.reseller.ResellerService;
 import com.openexchange.server.ServiceExceptionCode;
 import com.openexchange.server.ServiceLookup;
 
@@ -391,6 +364,22 @@ public class DataExportCleanUpTask implements Runnable {
             LOG.warn("Failed to retrieve possible identifiers of data export file storage from context-sets scope", e);
         }
 
+        // Check for possible configured file storage identifiers for reseller scope
+        try {
+            ResellerService resellerService = services.getOptionalService(ResellerService.class);
+            if (resellerService != null && resellerService.isEnabled()) {
+                Set<Integer> resellerFileStoreIds = determineResellerFileStoreIds(databaseService);
+                if (resellerFileStoreIds.isEmpty() == false) {
+                    if (fileStoreIds == null) {
+                        fileStoreIds = new HashSet<>();
+                    }
+                    fileStoreIds.addAll(resellerFileStoreIds);
+                }
+            }
+        } catch (Exception e) {
+            LOG.warn("Failed to retrieve possible identifiers of data export file storage from reseller scope", e);
+        }
+
         // Check for possible configured file storage identifiers for server scope
         try {
             ConfigurationService configurationService = services.getOptionalService(ConfigurationService.class);
@@ -470,6 +459,39 @@ public class DataExportCleanUpTask implements Runnable {
         return fileStoreIds == null ? Collections.emptySet() : fileStoreIds;
     }
 
+    private Set<Integer> determineResellerFileStoreIds(DatabaseService databaseService) throws OXException {
+        PreparedStatement stmt = null;
+        ResultSet rs = null;
+        Connection con = databaseService.getReadOnly();
+        try {
+            if (Databases.tableExists(con, "subadmin_config_properties") == false) {
+                return Collections.emptySet();
+            }
+
+            stmt = con.prepareStatement("SELECT propertyValue FROM subadmin_config_properties WHERE propertyKey = ?");
+            stmt.setString(1, "com.openexchange.gdpr.dataexport.fileStorageId");
+            rs = stmt.executeQuery();
+            if (rs.next() == false) {
+                return Collections.emptySet();
+            }
+
+            Set<Integer> fileStoreIds = new HashSet<>();
+            do {
+                try {
+                    fileStoreIds.add(Integer.valueOf(rs.getString(1)));
+                } catch (NumberFormatException e) {
+                    // Ignore
+                }
+            } while (rs.next());
+            return fileStoreIds;
+        } catch (SQLException e) {
+            throw DataExportExceptionCode.SQL_ERROR.create(e, e.getMessage());
+        } finally {
+            Databases.closeSQLStuff(rs, stmt);
+            databaseService.backReadOnly(con);
+        }
+    }
+
     private Set<Integer> determineFileStoreIdsFor(int representativeContextId, DatabaseService databaseService) throws OXException {
         PreparedStatement stmt = null;
         ResultSet rs = null;
