@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH. group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -66,6 +38,7 @@ import com.openexchange.config.cascade.ConfigViews;
 import com.openexchange.context.ContextService;
 import com.openexchange.database.CreateTableService;
 import com.openexchange.database.DatabaseService;
+import com.openexchange.exception.OXException;
 import com.openexchange.filestore.DatabaseAccessProvider;
 import com.openexchange.filestore.FileStorageService;
 import com.openexchange.gdpr.dataexport.DataExportConfig;
@@ -96,10 +69,13 @@ import com.openexchange.java.Strings;
 import com.openexchange.notification.mail.NotificationMailFactory;
 import com.openexchange.osgi.HousekeepingActivator;
 import com.openexchange.reseller.ResellerService;
+import com.openexchange.server.ServiceLookup;
 import com.openexchange.serverconfig.ServerConfigService;
 import com.openexchange.session.Session;
 import com.openexchange.threadpool.ThreadPoolService;
 import com.openexchange.timer.TimerService;
+import com.openexchange.tools.session.ServerSession;
+import com.openexchange.user.User;
 import com.openexchange.user.UserService;
 
 /**
@@ -215,6 +191,9 @@ public class DataExportActivator extends HousekeepingActivator {
             boolean replaceUnicodeWithAscii = configService.getBoolProperty("com.openexchange.gdpr.dataexport.replaceUnicodeWithAscii", false);
             configBuilder.withReplaceUnicodeWithAscii(replaceUnicodeWithAscii);
 
+            boolean allowPausingRunningTasks = configService.getBoolProperty("com.openexchange.gdpr.dataexport.allowPausingRunningTasks", false);
+            configBuilder.withAllowPausingRunningTasks(allowPausingRunningTasks);
+
             config = configBuilder.build();
         }
 
@@ -255,6 +234,7 @@ public class DataExportActivator extends HousekeepingActivator {
 
         // Announce GDPR data export available
         {
+            final ServiceLookup services = this;
             final String sCapability = "dataexport";
             Dictionary<String, Object> properties = new Hashtable<String, Object>(1);
             properties.put(CapabilityChecker.PROPERTY_CAPABILITIES, sCapability);
@@ -274,6 +254,11 @@ public class DataExportActivator extends HousekeepingActivator {
                             if (!enabled) {
                                 return FailureAwareCapabilityChecker.Result.DISABLED;
                             }
+
+                            User user = getUserFor(session, services);
+                            if (user.isAnonymousGuest() || user.isGuest()) {
+                                return FailureAwareCapabilityChecker.Result.DISABLED;
+                            }
                         } catch (Exception e) {
                             LOG.warn("Failed to check if GDPR data export is enabled for user {} in context {}", I(session.getUserId()), I(session.getContextId()), e);
                             return FailureAwareCapabilityChecker.Result.FAILURE;
@@ -282,6 +267,11 @@ public class DataExportActivator extends HousekeepingActivator {
 
                     return FailureAwareCapabilityChecker.Result.ENABLED;
                 }
+
+                private User getUserFor(Session session, ServiceLookup services) throws OXException {
+                    return session instanceof ServerSession ? ((ServerSession) session).getUser() : services.getServiceSafe(UserService.class).getUser(session.getUserId(), session.getContextId());
+                }
+
             }, properties);
             getService(CapabilityService.class).declareCapability(sCapability);
         }
