@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH. group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -63,6 +35,8 @@ import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Optional;
+import java.util.concurrent.Callable;
+import java.util.concurrent.TimeUnit;
 import java.util.function.Predicate;
 import java.util.stream.Collectors;
 import org.json.JSONException;
@@ -87,9 +61,11 @@ import com.openexchange.gdpr.dataexport.provider.mail.MailDataExportPropertyName
 import com.openexchange.gdpr.dataexport.provider.mail.generator.FailedAuthenticationResult;
 import com.openexchange.gdpr.dataexport.provider.mail.generator.SessionGenerator;
 import com.openexchange.java.Collators;
+import com.openexchange.java.Reference;
 import com.openexchange.java.Streams;
 import com.openexchange.java.Strings;
 import com.openexchange.java.util.UUIDs;
+import com.openexchange.mail.MailExceptionCode;
 import com.openexchange.mail.MailField;
 import com.openexchange.mail.MailSortField;
 import com.openexchange.mail.OrderDirection;
@@ -99,9 +75,15 @@ import com.openexchange.mail.api.IMailMessageStorage;
 import com.openexchange.mail.api.MailAccess;
 import com.openexchange.mail.dataobjects.MailFolder;
 import com.openexchange.mail.dataobjects.MailMessage;
+import com.openexchange.mail.mime.MimeMailExceptionCode;
 import com.openexchange.mail.mime.utils.MimeMessageUtility;
 import com.openexchange.mail.service.MailService;
 import com.openexchange.server.ServiceLookup;
+import com.openexchange.session.Session;
+import net.jodah.failsafe.Failsafe;
+import net.jodah.failsafe.FailsafeException;
+import net.jodah.failsafe.RetryPolicy;
+import net.jodah.failsafe.function.CheckedConsumer;
 
 /**
  * {@link MailDataExport}
@@ -111,7 +93,8 @@ import com.openexchange.server.ServiceLookup;
  */
 public class MailDataExport extends AbstractDataExportProviderTask {
 
-    private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(MailDataExport.class);
+    /** The logger constant */
+    static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(MailDataExport.class);
 
     private static final String ID_MAIL = "mail";
 
@@ -158,7 +141,7 @@ public class MailDataExport extends AbstractDataExportProviderTask {
     public ExportResult export() throws OXException {
         MailService mailService = services.getServiceSafe(MailService.class);
 
-        MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> mailAccess = null;
+        Reference<MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage>> mailAccessReference = new Reference<>();
         try {
             Module mailModule = getModule();
 
@@ -193,8 +176,8 @@ public class MailDataExport extends AbstractDataExportProviderTask {
             boolean tryConnect = true;
             while (tryConnect) {
                 try {
-                    mailAccess = mailService.getMailAccess(session, 0);
-                    mailAccess.connect();
+                    mailAccessReference.setValue(mailService.getMailAccess(session, 0));
+                    mailAccessReference.getValue().connect();
                     tryConnect = false;
                 } catch (OXException e) {
                     // Check for failed authentication
@@ -218,7 +201,7 @@ public class MailDataExport extends AbstractDataExportProviderTask {
 
             Folder rootFolder;
             {
-                IMailFolderStorage folderStorage = mailAccess.getFolderStorage();
+                @SuppressWarnings("null") IMailFolderStorage folderStorage = mailAccessReference.getValue().getFolderStorage(); // mailAccess is guarded by 'tryConnect'
                 IMailFolderStorageInfoSupport infoSupport = folderStorage.supports(IMailFolderStorageInfoSupport.class);
                 if (null != infoSupport && infoSupport.isInfoSupported()) {
                     rootFolder = new DefaultFolder(infoSupport.getFolderInfo(MailFolder.ROOT_FOLDER_ID), infoSupport);
@@ -227,14 +210,14 @@ public class MailDataExport extends AbstractDataExportProviderTask {
                 }
             }
 
-            mailAccess.setWaiting(true); // Mark as "waiting" to prevent from mail access watcher outputting false-positive warnings
+            mailAccessReference.getValue().setWaiting(true); // Mark as "waiting" to prevent from mail access watcher outputting false-positive warnings
             try {
-                SavePointAndReason optSavePoint = traverseFolder(rootFolder, null, startInfo, options, mailAccess);
+                SavePointAndReason optSavePoint = traverseFolder(rootFolder, null, startInfo, options, mailAccessReference);
                 if (optSavePoint != null) {
                     return optSavePoint.result();
                 }
             } finally {
-                mailAccess.setWaiting(false); // Unset "waiting" flag
+                mailAccessReference.getValue().setWaiting(false); // Unset "waiting" flag
             }
 
             tryTouch();
@@ -254,11 +237,11 @@ public class MailDataExport extends AbstractDataExportProviderTask {
             }
             throw OXException.general(e.getMessage(), e);
         } finally {
-            MailAccess.closeInstance(mailAccess);
+            MailAccess.closeInstance(mailAccessReference.getValue());
         }
     }
 
-    private SavePointAndReason traverseFolder(Folder folder, String parentPath, StartInfo startInfo, Options options, MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> mailAccess) throws OXException, DataExportAbortedException {
+    private SavePointAndReason traverseFolder(Folder folder, String parentPath, StartInfo startInfo, Options options, Reference<MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage>> mailAccessReference) throws OXException, DataExportAbortedException {
         if (isPauseRequested()) {
             if (startInfo != null) {
                 JSONObject jSavePoint = new JSONObject(4);
@@ -290,7 +273,7 @@ public class MailDataExport extends AbstractDataExportProviderTask {
             }
 
             if (folder.isHoldsMessages()) {
-                SavePointAndReason jSavePoint = exportMessages(folder.getFullname(), pathOfFolder, info == null ? null : info.mailId, mailAccess);
+                SavePointAndReason jSavePoint = exportMessages(folder.getFullname(), pathOfFolder, info == null ? null : info.mailId, mailAccessReference);
                 if (jSavePoint != null) {
                     return jSavePoint;
                 }
@@ -308,7 +291,7 @@ public class MailDataExport extends AbstractDataExportProviderTask {
                     JSONObject jSavePoint = new JSONObject(4);
                     jSavePoint.putSafe("folder", info.fullName);
                     if (info.path != null) {
-                        jSavePoint.putSafe("path", startInfo.path);
+                        jSavePoint.putSafe("path", info.path);
                     }
                     if (info.mailId != null) {
                         jSavePoint.putSafe("id", info.mailId);
@@ -334,7 +317,7 @@ public class MailDataExport extends AbstractDataExportProviderTask {
                 }
             }
             for (Folder child : children) {
-                SavePointAndReason jSavePoint = traverseFolder(child, pathOfFolder, info, options, mailAccess);
+                SavePointAndReason jSavePoint = traverseFolder(child, pathOfFolder, info, options, mailAccessReference);
                 if (jSavePoint != null) {
                     return jSavePoint;
                 }
@@ -354,7 +337,7 @@ public class MailDataExport extends AbstractDataExportProviderTask {
 
     private static final MailField[] FIELDS_ID = new MailField[] { MailField.ID };
 
-    private SavePointAndReason exportMessages(String fullName, String path, String startMailId, MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> mailAccess) throws OXException, DataExportAbortedException {
+    private SavePointAndReason exportMessages(String fullName, String path, String startMailId, Reference<MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage>> mailAccessReference) throws OXException, DataExportAbortedException {
         if (isPauseRequested()) {
             JSONObject jSavePoint = new JSONObject(4);
             jSavePoint.putSafe("folder", fullName).putSafe("path", path);
@@ -367,7 +350,7 @@ public class MailDataExport extends AbstractDataExportProviderTask {
 
         MailMessage[] messages;
         try {
-            messages = mailAccess.getMessageStorage().getAllMessages(fullName, null, MailSortField.RECEIVED_DATE, OrderDirection.ASC, FIELDS_ID);
+            messages = mailAccessReference.getValue().getMessageStorage().getAllMessages(fullName, null, MailSortField.RECEIVED_DATE, OrderDirection.ASC, FIELDS_ID);
             if (messages.length <= 0) {
                 // No messages in given folder
                 return null;
@@ -429,6 +412,36 @@ public class MailDataExport extends AbstractDataExportProviderTask {
             return null;
         }
 
+        ServiceLookup services = this.services;
+        CheckedConsumer<Throwable> onRetryConsumer = new CheckedConsumer<Throwable>() {
+
+            @Override
+            public void accept(Throwable t) throws Exception {
+                MailService mailService = services.getServiceSafe(MailService.class);
+
+                MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> mailAccess = null;
+                try {
+                    // Grab session associated with current mail access in order to establish a new one
+                    Session session = mailAccessReference.getValue().getSession();
+
+                    // Drop old mail access
+                    mailAccessReference.getValue().setWaiting(false);
+                    MailAccess.closeInstance(mailAccessReference.getAndSetValue(null), false);
+
+                    // Establish new mail access
+                    mailAccess = mailService.getMailAccess(session, 0);
+                    mailAccess.connect();
+                    mailAccess.setWaiting(true); // Mark as "waiting" to prevent from mail access watcher outputting false-positive warnings
+                    mailAccessReference.setValue(mailAccess);
+                    mailAccess = null; // Avoid premature closing
+                } finally {
+                    MailAccess.closeInstance(mailAccess);
+                }
+
+                LOG.debug("Connection to mail server closed. Retrying...", t);
+            }
+        };
+
         int batchCount = 0;
         for (MailMessage message : messages) {
             String mailId = message.getMailId();
@@ -449,12 +462,10 @@ public class MailDataExport extends AbstractDataExportProviderTask {
 
             InputStream stream = null;
             try {
-                MailMessage m = mailAccess.getMessageStorage().getMessage(fullName, mailId, false);
-                if (m == null) {
-                    LOG.debug("No such mail {} ({} of {}) in directory {} for data export {} of user {} in context {}", mailId, I(batchCount), I(messages.length), fullName, UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()));
-                } else {
-                    stream = MimeMessageUtility.getStreamFromMailPart(m);
-                    boolean exported = sink.export(stream, new Item(path, sanitizeNameForZipEntry(mailId + ".eml"), m.getSentDate()));
+                StreamFromMailPartCallable streamFromMailPartCallable = new StreamFromMailPartCallable(messages.length, batchCount, mailAccessReference, mailId, fullName, task);
+                stream = executeWithRetryOnConnectionClosure(streamFromMailPartCallable, onRetryConsumer);
+                if (stream != null) {
+                    boolean exported = sink.export(stream, new Item(path, sanitizeNameForZipEntry(mailId + ".eml"), streamFromMailPartCallable.getSentDate()));
                     if (!exported) {
                         return savePointFor(new JSONObject(4).putSafe("folder", fullName).putSafe("path", path).putSafe("id", mailId));
                     }
@@ -474,6 +485,97 @@ public class MailDataExport extends AbstractDataExportProviderTask {
         return null;
     }
 
+    private static class StreamFromMailPartCallable implements Callable<InputStream> {
+
+        private final int messagesLength;
+        private final int currentBatchCount;
+        private final String mailId;
+        private final String fullName;
+        private final DataExportTask task;
+        private final Reference<MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage>> mailAccessReference;
+        private Date sentDate;
+
+        StreamFromMailPartCallable(int messagesLength, int currentBatchCount, Reference<MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage>> mailAccessReference, String mailId, String fullName, DataExportTask task) {
+            super();
+            this.messagesLength = messagesLength;
+            this.currentBatchCount = currentBatchCount;
+            this.mailAccessReference = mailAccessReference;
+            this.mailId = mailId;
+            this.fullName = fullName;
+            this.task = task;
+        }
+
+        @Override
+        public InputStream call() throws OXException {
+            MailMessage m = mailAccessReference.getValue().getMessageStorage().getMessage(fullName, mailId, false);
+            if (m == null) {
+                LOG.debug("No such mail {} ({} of {}) in directory {} for data export {} of user {} in context {}", mailId, I(currentBatchCount), I(messagesLength), fullName, UUIDs.getUnformattedString(task.getId()), I(task.getUserId()), I(task.getContextId()));
+                return null;
+            }
+
+            sentDate = m.getSentDate();
+            return MimeMessageUtility.getStreamFromMailPart(m);
+        }
+
+        /**
+         * Gets the sent date of the message.
+         *
+         * @return The sent date
+         */
+        Date getSentDate() {
+            return sentDate;
+        }
+    }
+
+    /**
+     * Executes given callable with one-time retry behavior on connect error.
+     *
+     * @param callable The callable to execute
+     * @param onRetryConsumer The retry consumer
+     * @return The return value
+     * @throws OXException If a non-retriable exception occurs or retry attempts are expired
+     */
+    static InputStream executeWithRetryOnConnectionClosure(StreamFromMailPartCallable callable, CheckedConsumer<Throwable> onRetryConsumer) throws OXException {
+        try {
+            return Failsafe.with(RETRY_POLICY_CONNECTION_CLOSED).onRetry(onRetryConsumer).get(callable);
+        } catch (FailsafeException e) {
+            // Checked exception occurred
+            Throwable cause = e.getCause();
+            if (cause instanceof OXException) {
+                throw (OXException) cause;
+            }
+
+            Throwable t = cause == null ? e : cause;
+            throw MailExceptionCode.UNEXPECTED_ERROR.create(t, t.getMessage());
+        } catch (IllegalArgumentException e) {
+            String message = e.getMessage();
+            if (message != null && message.startsWith("Invalid fully-qualifying mail folder identifier: ")) {
+                // Apparently, passed folder identifier is invalid
+                throw MailExceptionCode.INVALID_FOLDER_IDENTIFIER.create(e, message.substring(49));
+            }
+            // Unchecked exception occurred
+            throw MailExceptionCode.UNEXPECTED_ERROR.create(e, e.getMessage());
+        } catch (RuntimeException e) {
+            // Unchecked exception occurred
+            throw MailExceptionCode.UNEXPECTED_ERROR.create(e, e.getMessage());
+        }
+    }
+
+    private static final RetryPolicy RETRY_POLICY_CONNECTION_CLOSED = new RetryPolicy()
+        .withMaxRetries(1).withBackoff(1, 10, TimeUnit.SECONDS).withJitter(0.25f)
+        .retryOn(t ->
+            {
+                if (!(OXException.class.isInstance(t))) {
+                    return false;
+                }
+
+                OXException oxe = (OXException) t;
+                return MimeMailExceptionCode.CONNECTION_CLOSED.equals(oxe)
+                    || MimeMailExceptionCode.CONNECT_ERROR.equals(oxe)
+                    || MimeMailExceptionCode.READ_TIMEOUT.equals(oxe);
+            }
+        );
+
     // ------------------------------------------------------------ Helpers -------------------------------------------------------------
 
     private GeneratedSession generateSession(Module mailModule, SessionGenerator sessionGenerator) throws OXException {
